local SETTINGS = {
	colors = {
		real = {r=1,g=0,b=0,a=1},
		energy = {r=0,g=1,b=1,a=1},
		aura = {r=0,g=1,b=0,a=1},
		default = {r=1,g=1,b=0,a=1},
	},
	font = STANDARD_TEXT_FONT,
	size = 18,
	glint = true,
}

local GetTime = GetTime
local _ -- A blizzard addon is using a global version of this variable and causing taint

local function fTime(s)
	if s >= 86400 then return '%dd',s/86400 end
	if s >= 3600 then return '%dh',s/3600 end
	if s >= 60 then return '%dm',s/60 end
	if s >= 10 then return '%d',s end
	return '%.1f',s
end

-- Glint animation handler --
local aniCount = 5
local duration = 0.2
local animations = {}
for i=1, aniCount do
	local frame = CreateFrame('Frame')
	local tex = frame:CreateTexture()
	tex:SetTexture([[Interface\Cooldown\star4]])
	tex:SetAlpha(0)
	tex:SetAllPoints()
	tex:SetBlendMode('ADD')
	local group = tex:CreateAnimationGroup()
	local alpha = group:CreateAnimation('Alpha')
	alpha:SetOrder(1)
	alpha:SetChange(1)
	alpha:SetDuration(0)
	local scale = group:CreateAnimation('Scale')
	scale:SetOrder(1)
	scale:SetScale(1.5,1.5)
	scale:SetDuration(0)
	local scale2 = group:CreateAnimation('Scale')
	scale2:SetOrder(2)
	scale2:SetScale(0,0)
	scale2:SetDuration(duration)
	local spin = group:CreateAnimation('Rotation')
	spin:SetOrder(2)
	spin:SetDegrees(90)
	spin:SetDuration(duration)
	animations[i] = {frame = frame, group = group}
end

local aniNum = 1
local function animate(button)
	if not SETTINGS.glint or not button:IsVisible() then return end
	local animation = animations[aniNum]
	local frame,group = animation.frame,animation.group
	group:Stop()
	frame:SetFrameStrata(button:GetFrameStrata())
	frame:SetFrameLevel(button:GetFrameLevel() + 10)
	frame:SetAllPoints(button)
	group:Play()
	aniNum = (aniNum % aniCount) + 1
end

-- Cooldown spinner and action timers --
local timers,actions = {},{}
local function CooldownTimer(parent)
	if timers[parent] then return timers[parent] end
	local timer = CreateFrame('frame', nil, parent)
	timer:SetAllPoints(parent)
	timer:SetFrameLevel(parent:GetFrameLevel() + 3)
	
	timer.text = timer:CreateFontString(nil, 'OVERLAY')
	--timer.text:SetShadowOffset(1,-1)
	timer.text:SetFont(SETTINGS.font or STANDARD_TEXT_FONT, SETTINGS.size or 18, 'OUTLINE')
	
	timer.start = 0
	timer.duration = 0
	
	timers[parent] = timer
	return timer
end

local function ActionTimer(parent)
	if actions[parent] then return actions[parent] end
	local timer = CreateFrame('frame', nil, parent)
	timer:SetAllPoints(parent)
	timer:SetFrameLevel(parent:GetFrameLevel() + 3)
	
	timer.text = timer:CreateFontString(nil, 'OVERLAY')
	--timer.text:SetShadowOffset(1,-1)
	timer.text:SetFont(SETTINGS.font or STANDARD_TEXT_FONT, SETTINGS.size or 18, 'OUTLINE')
	timer.text:SetPoint('CENTER')
	
	timer.border = timer:CreateTexture(nil, 'BACKGROUND')
	timer.border:SetTexture([[Interface\Buttons\CheckButtonHilight]])
	timer.border:SetAllPoints()
	timer.border:SetBlendMode('add')
	--t:SetVertexColor(0, 0.25, 1, 1)
	
	--timer.action = parent:GetAttribute('action') or ActionButton_CalculateAction(parent)
	timer.action = parent:GetAttribute('action') or  ActionButton_GetPagedID(parent) or ActionButton_CalculateAction(parent)
	
	actions[parent] = timer
	return timer
end

local function SetTimer(parent, start, duration, action)
	if not parent then return end
	local timer = CooldownTimer(parent)
	timer.start = start or 0
	timer.duration = duration or 0
end

-- Fires normally when a cooldown spinner is set, except on action buttons, which behave spuriously
hooksecurefunc(getmetatable(ActionButton1.cooldown).__index, 'SetCooldown', function(cooldown, start, duration, enable)
	if cooldown:GetParent():GetAttribute('type') == 'action' then return end
	SetTimer(cooldown, start, duration, 'spinner')
end)

-- Hook for action buttons created by an addon like bartender
hooksecurefunc(getmetatable(ActionButton1).__index, 'SetAttribute', function(button, attribute, slot)
	if attribute ~= 'action' or not HasAction(slot) then return end
	if not actions[button] then
		ActionTimer(button)
	end
	actions[button].action = slot
end)

-- Loop through every frame in the interface to register existing action buttons
local function GetActionButtons(...)
	for i = 1, select('#',...) do
		local frame = select(i, ...)
		if frame:GetNumChildren() > 0 then
			GetActionButtons(frame:GetChildren())
		end
		if frame:GetAttribute('type') == 'action' then
			ActionTimer(frame)
		end
	end
end
--GetActionButtons(UIParent:GetChildren())

-- Play Glint animation on any action buttons that match
hooksecurefunc('UseAction', function(slot1, target, button)
	for button,timer in pairs(actions) do
		--local slot2 = button:GetAttribute('action') or  ActionButton_GetPagedID(button) or ActionButton_CalculateAction(button)
		if slot1 == timer.action and button:IsVisible() then
			animate(button)
		end
	end
end)


local function SetTimerTime(timer,left,color)
	local width = timer:GetWidth() or 0
	if not timer:IsVisible() or width <= 0 or left <= 0 then return end
	if timer.border and color then
		if IsUsableAction(timer.action) and color == SETTINGS.colors.aura then
			timer.border:SetVertexColor(SETTINGS.colors.aura.r,SETTINGS.colors.aura.g,SETTINGS.colors.aura.b,SETTINGS.colors.aura.a)
		else
			timer.border:SetVertexColor(SETTINGS.colors.real.r,SETTINGS.colors.real.g,SETTINGS.colors.real.b,SETTINGS.colors.real.a)
		end
	end
	if not color then color = SETTINGS.colors.default end
	timer.text:SetTextColor(color.r or 1, color.g or 1, color.b or 0, color.a or 1)
	timer.text:ClearAllPoints()
	-- todo: only resize font strings when the frame resizes instead of setting the font every single update
	-- :HookScript('OnSizeChanged', fixFonts)
	if width < 26 then
		if left < 60 then
			timer.text:SetPoint("CENTER", timer, "TOPRIGHT")
			timer.text:SetFont(SETTINGS.font or STANDARD_TEXT_FONT, width*(SETTINGS.size/26), "OUTLINE")
			timer.text:SetFormattedText("%d", left)
		else
			timer.text:SetText()
		end
	else
		timer.text:SetPoint('CENTER')
		if left <= 600 then
			timer.text:SetFont(SETTINGS.font or STANDARD_TEXT_FONT, width*(SETTINGS.size/36), "OUTLINE")
		else
			timer.text:SetFont(SETTINGS.font or STANDARD_TEXT_FONT, width*(SETTINGS.size/45), "OUTLINE")
		end
		timer.text:SetFormattedText(fTime(left))
	end
end


--\\---------------------------\\--
--//   MANY AURAS, HANDLE IT   //--
--\\---------------------------\\--
local GetSpellInfo = GetSpellInfo
local auras = {} -- [aura name] = expiration time
local SharedAurasList = { -- [aura name] = psuedo group
	-- Buffs -->>
	-->> +5% str,agi,int
	[1126] = '+stats', -- Mark of the Wild (Druid)
	[115921] = '+stats', -- Legacy of the Emperor (Monk)
	[20217] = '+stats', -- Blessing of Kings (Paladin)
	[90363] = '+stats', -- Embrace of the Shale Spider (Hunter, Shale Spider)
	-->> +10% stamina
	[21562] = '+stam', -- Power Word: Fortitude (Priest)
	[103127] = '+stam', -- Blood Pact (Warlock, Imp)
	[469] = '+stam', -- Commanding Shout (Warrior)
	[90364] = '+stam', -- Qiraji Fortitude (Hunter, Silithid)
	-->> +10% attack power
	[57330] = '+ap', -- Horn of Winter (Death Knight)
	[19506] = '+ap', -- Trueshot Aura (Hunter)
	[6673] = '+ap', -- Battle Shout (Warrior)
	-->> +10% spell power
	[1459] = '+sp', -- Arcane Brilliance (Mage) (Also grants crit)
	[61316] = '+sp', -- Dalaran Brilliance (Mage) (Also grants crit)
	[77747] = '+sp', -- Burning Wrath (Shaman)
	[109773] = '+sp', -- Dark Intent (Warlock)
	[126309] = '+sp', -- Still Water (Hunter, Waterstrider) (Also grants crit)
	-->> +10% haste
	[55610] = '+haste', -- Unholy Aura (Death Knight)
	[113742] = '+haste', -- Swiftblade's Cunning (Rogue)
	[30809] = '+haste', -- Unleashed Rage (Shaman)
	[128432] = '+haste', -- Cackling Howl (Hunter, Hyena)
	[128433] = '+haste', -- Serpent's Swiftness (Hunter, Serpent)
	-->> +5% spell haste
	[24907] = '+sphaste', -- Moonkin Aura (Druid)
	[15473] = '+sphaste', -- Shadowform (Priest) (Is this what the group buff is actually called?)
	[51470] = '+sphaste', -- Elemental Oath (Shaman)
	-->> +5% crit
	[17007] = '+crit', -- Leader of the Pack (Druid)
	--[1459] = '+sp', -- Arcane Brilliance (Mage) (Also grants spell power)
	--[61316] = '+sp', -- Dalaran Brilliance (Mage) (Also grants spell power)
	[116781] = '+crit', -- Legacy of the White Tiger (Monk)
	[97229] = '+crit', -- Bellowing Roar (Hunter, Hydra)
	[24604] = '+crit', -- Furious Howl (Hunter, Wolf)
	[90309] = '+crit', -- Terrifying Roar (Hunter, Devilsaur)
	[126373] = '+crit', -- Fearless Roar (Hunter, Quilen)
	--[126309] = '+crit', -- Still Water (Hunter, Waterstrider) (Also grants spell power)
	-->> +Mastery
	[19740] = '+mastery', -- Blessing of Might (Paladin)
	[116956] = '+mastery', -- Grace of Air (Shaman)
	[93435] = '+mastery', -- Roar of Courage (Hunter, Cat)
	[128997] = '+mastery', -- Spirit Beast Blessing (Hunter, Spirit Beast)
	
	-- Debuffs -->>
	[1130] = 1130, -- Hunter's Mark (Hunter)
	-->> -12% armor
	[113746] = '-armor', -- Weakened Armor (General Debuff)
	[770] = '-armor', -- Faerie Fire (Druid)
	[8647] = '-armor', -- Expose Armor (Rogue)
	[7386] = '-armor', -- Sunder Armor (Warrior)
	[50285] = '-armor', -- Dust Cloud (Hunter, Tallstrider)
	[50498] = '-armor', -- Tear Armor (Hunter, Raptor)
	-->> +4% damage taken
	[81326] = '+pain', -- Physical Vulnerability (General Debuff)
	[81328] = '+pain', -- Brittle Bones (Death Knight) (unsure)
	[51160] = '+pain', -- Ebon Plaguebringer (Death Knight) (unsure)
	[111529] = '+pain', -- Judgments of the Bold (Paladin)
	--[86346] = '+pain', -- Colossus Smash (Warrior) (maybe ignore this)
	[35290] = '+pain', -- Gore (Hunter, Boar)
	[50518] = '+pain', -- Ravage (Hunter, Ravager)
	[57386] = '+pain', -- Stampede (Hunter, Rhino)
	[55746] = '+pain', -- Acid Spit (Hunter, Worm)
	-->> +8% spell damage taken (Does this have a general debuff?)
	[1490] = '+spellpain', -- Curse of the Elements (Warlock)
	[58410] = '+spellpain', -- Master Poisoner (Rogue)
	[34889] = '+spellpain', -- Fire Breath (Hunter, Fire Breath)
	[24844] = '+spellpain', -- Lightning Breath (Hunter, Wind Serpent)
	-->> -10% physical damage
	[115798] = '+weak', -- Weakened Blows (General Debuff)
	[81132] = '+weak', -- Scarlet Fever (Death Knight)
	[106830] = '+weak', -- Thrash (Druid)
	[121253] = '+weak', -- Keg Smash (Monk)
	[53595] = '+weak', -- Hammer of the Righteous (Paladin)
	[8042] = '+weak', -- Earth Shock (Shaman)
	[6343] = '+weak', -- Thunder Clap (Warrior)
	[50256] = '+weak', -- Demoralizing Roar (Hunter, Bear)
	[24423] = '+weak', -- Demoralizing Screech (Hunter, Carrion Bird)
	-->> -30% casting speed
	[73975] = '-cast', -- Necrotic Strike (Death Knight)
	[31589] = '-cast', -- Slow (Mage)
	[5761] = '-cast', -- Mind-numbing Poison (Rogue)
	[109466] = '-cast', -- Curse of Enfeeblement (Warlock)
	[50274] = '-cast', -- Spore Cloud (Hunter, Sporebat)
	[90314] = '-cast', -- Tailspin (Hunter, Fox)
	[126402] = '-cast', -- Trample (Hunter, Goat)
	[58604] = '-cast', -- Lava Breath (Hunter, Core Hound)
	-->> -25% healing taken
	[115804] = '-heals', -- Mortan Wounds (General Debuff)
	[12294] = '-heals', -- Mortal Strike (Warrior)
	[100130] = '-heals', -- Wild Strike (Warrior)
	[8679] = '-heals', -- Wound Poison (Rogue)
	[82654] = '-heals', -- Widow Venom (Hunter)
	[107428] = '-heals', -- Rising Sun Kick (Monk)
	[54680] = '-heals', -- Monstrous Bite (Hunter, Devilsaur)
	
	-->> Crowd Controls
	-- Druid
	[2637] = 'CC', -- Hibernate
	--[339] = 'CC', -- Entangling Roots
	--[33786] = 'CC', -- Cyclone
	-- Hunter
	[3355] = 'CC', -- Freezing Trap Debuff
	[1499] = 'CC', -- Freezing Trap Ability
	[60192] = 'CC',  -- Freezing Trap Launcher Ability
	[1513] = 'CC', -- Scare Beast
	[19386] = 'CC', -- Wyvern Sting
	-- Warlock
	[5782] = 'CC', -- Fear
	[710] = 'CC',  -- Banish
	[6358] = 'CC', -- Seduction
	-- Mage
	[118] = 'CC', -- Polymorph (Sheep)
	[61305] = 'CC', -- Polymorph (Black Cat)
	[28272] = 'CC', -- Polymorph (Pig)
	[61721] = 'CC', -- Polymorph (Rabbit)
	[61780] = 'CC', -- Polymorph (Turkey)
	[28271] = 'CC', -- Polymorph (Turtle)
	-- Paladin
	[20066] = 'CC', -- Repentance
	[10326] = 'CC', -- Turn Evil
	-- Rogue
	[6770] = 'CC', -- Sap
	[2094] = 'CC', -- Blind
	-- Shaman
	[51514] = 'CC',  -- Hex
	[76780] = 'CC', -- Bind Elemental
	-- Priest
	[9484] = 'CC', -- Shackle Undead
	-- Monk
	[115078] = 'CC', -- Paralysis
	
	-->> Disarm
	[50541] = 'disarmed', -- Clench (Hunter, Scorpid)
	[676] = 'disarmed', -- Disarm (Warrior)
	[51722] = 'disarmed', -- Dismantle (Rogue)
	[64058] = 'disarmed', -- Psychic Horror (Priest)
	[91644] = 'disarmed', -- Snatch (Hunter, Bird of Prey)
	[117368] = 'disarmed', -- Grapple Weapon (Monk)
	
	-->> Silenced
	[28730] = 'silenced', -- Arcane Torrent (Mana)
	[50613] = 'silenced', -- Arcane Torrent (Runic Power)
	[80483] = 'silenced', -- Arcane Torrent (Focus)
	[25046] = 'silenced', -- Arcane Torrent (Energy)
	[69179] = 'silenced', -- Arcane Torrent (Rage)
	[31935] = 'silenced', -- Avenger's Shield (Paladin)
	[1330] = 'silenced', -- Garrote - Silence (Rogue)
	[50479] = 'silenced', -- Nether Shock (Hunter: Nether Ray)
	[15487] = 'silenced', -- Silence (Priest)
	[18498] = 'silenced', -- Silenced - Gag Order (Warrior)
	[18469] = 'silenced', -- Silenced - Improved Counterspell (Mage)
	[18425] = 'silenced', -- Silenced - Improved Kick (Rogue)
	[34490] = 'silenced', -- Silencing Shot (Hunter)
	[81261] = 'silenced', -- Solar Beam (Druid)
	[24259] = 'silenced', -- Spell Lock (Warlock: Fel Hunter)
	[47476] = 'silenced', -- Strangulate (Death Knight)
	
	-->> Stun
	[85387] = 'stunned', -- Aftermath (Warlock)
	[89766] = 'stunned', -- Axe Toss (Warlock: Felguard)
	[5211] = 'stunned', -- Bash (Druid)
	[93433] = 'stunned', -- Burrow Attack (Hunter: Worm)
	[7922] = 'stunned', -- Charge Stun (Warrior)
	[1833] = 'stunned', -- Cheap Shot (Rogue)
	[12809] = 'stunned', -- Concussion Blow (Warrior)
	[44572] = 'stunned', -- Deep Freeze (Mage)
	[45334] = 'stunned', -- Feral Charge (Bear) (Druid)
	[91800] = 'stunned', -- Gnaw (Death Knight: Ghoul)
	[853] = 'stunned', -- Hammer of Justice (Paladin)
	[88625] = 'stunned', -- Holy Word: Chastise (Priest)
	[2812] = 'stunned', -- Holy Wrath (Paladin)
	[12355] = 'stunned', -- Impact (Mage)
	[20253] = 'stunned', -- Intercept (Warrior)
	[19577] = 'stunned', -- Intimidation (Hunter)
	[408] = 'stunned', -- Kidney Shot (Rogue)
	[22570] = 'stunned', -- Maim (Druid)
	[9005] = 'stunned', -- Pounce (Druid)
	[82691] = 'stunned', -- Ring of Frost (Mage)
	[30283] = 'stunned', -- Shadowfury (Warlock)
	[46968] = 'stunned', -- Shockwave (Warrior)
	[50519] = 'stunned', -- Sonic Blast (Hunter: Bat)
	[56626] = 'stunned', -- Sting (Hunter: Wasp)
	[85388] = 'stunned', -- Throwdown (Warrior)
	[20549] = 'stunned', -- War Stomp (Tauren racial)
	[102795] = 'stunned', -- Bear Hug (Druid)
	[117418] = 'stunned', -- Fists of Fury (Monk)
}
local SharedAuras = {}
for spellID,aura in pairs(SharedAurasList) do -- Fill up table with spell names instead of IDs so I don't have to track down every last ID for the same spell
	local spellName = GetSpellInfo(spellID)
	if spellName then
		SharedAuras[spellName] = aura
	end
end

local StickyAuras = {} -- things to keep track of regardless of target, like my crowd controls, single-target abilities
local function updateAuras()
	wipe(auras) -- lazy
	local unit = (UnitExists("target") and (UnitIsPlayer("target") or UnitCanAttack("player", "target"))) and "target" or "player"
	if UnitIsFriend("player", unit) then
		for i = 1,40 do
			local name, _, _, _, _, _, expires, caster = UnitBuff(unit, i)
			--local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff, value1, value2, value3 = UnitBuff(unit, i)
			if not name then break end
			if caster == "player" and expires > 0 then
				if (auras[name] and expires > auras[name]) or not auras[name] then
					auras[name] = expires
				end
			elseif SharedAuras[name] and expires > 0 then
				auras[SharedAuras[name]] = expires
			end
		end
		for i = 1,40 do
			local name, _, _, _, _, _, expires, caster, _, _, spellID = UnitBuff("player", i)
			if not name then break end
			if caster == "player" and expires > 0 and SpellIsSelfBuff(spellID) then
				if name == 'Tiger Power' then name = 'Tiger Palm' end
				if (auras[name] and expires > auras[name]) or not auras[name] then
					auras[name] = expires
				end
			end
		end
	else
		for i = 1,40 do
			local name, _, _, _, _, _, expires, caster = UnitDebuff(unit, i)
			if not name then break end
			if caster == "player" and expires > 0 then
				if (auras[name] and expires > auras[name]) or not auras[name] then
					auras[name] = expires
				end
			elseif SharedAuras[name] and expires > 0 then
				auras[SharedAuras[name]] = expires
			end
		end
		for i = 1,40 do
			local name, _, _, _, _, _, expires, caster = UnitBuff("player", i)
			if not name then break end
			if caster == "player" and expires > 0 then
				if name == 'Tiger Power' then name = 'Tiger Palm' end
				if (auras[name] and expires > auras[name]) or not auras[name] then
					auras[name] = expires
				end
			elseif SharedAuras[name] and expires > 0 then
				auras[SharedAuras[name]] = expires
			end
		end
	end
end

--[[
local _,CLASS = UnitClass('player')
local function GetTotemCooldown(totem)
	for i=1,4 do
		local haveTotem, name, startTime, duration = GetTotemInfo(i)
		if haveTotem and name == totem then
			return (startTime + duration) or 0
		end
	end
	return 0
end
--]]

local f,ts = CreateFrame('frame'),0
f:SetScript('OnUpdate', function(self, e)
	ts = ts + e
	if ts >= 0.1 then
		local now,left,parent,timer = GetTime(),0
		for parent,timer in pairs(timers) do
			if timer.duration > 2.2 then
				left = timer.start + timer.duration - now
				SetTimerTime(timer,left)
			end
		end
		for button,timer in pairs(actions) do
			--local slot = timer.action or button:GetAttribute('action') or  ActionButton_GetPagedID(button) or ActionButton_CalculateAction(button)
			local slot = button:GetAttribute('action') or  ActionButton_GetPagedID(button) or ActionButton_CalculateAction(button)
			timer.action = slot
			if HasAction(slot) and button:IsVisible() then
				local start, duration, enabled = GetActionCooldown(slot)
				
				local realCooldown, energyCooldown, auraDuration = 0, 0, 0
				local btype, id = GetActionInfo(slot)
				if btype == 'macro' then _,_,id = GetMacroSpell(id) end -- tainting the global underscore, could use select() here
				if btype == 'item' then
					local name = GetItemInfo(id)
					if name and auras[name] then
						auraDuration = auras[name]-now
					end
				elseif btype == 'spell' or (btype == 'macro' and id) then
					local name, _, _, cost, _, power = GetSpellInfo(id)
					if name then
						if auras[name] then
							auraDuration = auras[name] - now
							--SpellIsSelfBuff(id)
						elseif SharedAuras[name] and auras[SharedAuras[name]] then
							auraDuration = auras[SharedAuras[name]] - now
						end
						
						--[[----------------------- Totem Timer Test - MultiCastActionButton doesn't update until a slot change, this could be a sign of a bigger issue
						if CLASS == 'SHAMAN' then
							local totemTime = GetTotemCooldown(name)
							if totemTime > 0 then
								auraDuration = totemTime - now
							end
						end
						--]]---------------------- Totem Timer Test
						
						if UnitPowerType("player") == power and cost > 0 then -- because GetPowerRegen only works for the active energy type
							local currentPower = UnitPower("player", power)
							if cost > currentPower then
								local passiveRegen, activeRegen = GetPowerRegen()
								local regenRate = UnitAffectingCombat("player") and activeRegen or passiveRegen
								local powerDifference = cost - currentPower
								if regenRate >= 1 then
									energyCooldown = powerDifference/regenRate
								end
							end
						end
					end
				end
				
				left = start + duration - now
				if left > energyCooldown and left > 0 and duration > 2.2 then
					if enabled == 1 then
						SetTimerTime(timer,left,SETTINGS.colors.real)
					else
						SetTimerTime(timer,left,{r=0.6,g=0.6,b=0.6,a=1})
					end
				elseif energyCooldown > 0 then
					SetTimerTime(timer,energyCooldown,SETTINGS.colors.energy)
				elseif auraDuration > 0 then
					SetTimerTime(timer,auraDuration,SETTINGS.colors.aura)
				else
					timer.text:SetText()
					timer.border:SetVertexColor(1,1,1,0)
				end
			else
				timer.text:SetText()
				timer.border:SetVertexColor(1,1,1,0)
			end
		end
		ts = 0
	end
end)

local function recalculateActions()
	for button,timer in pairs(actions) do
		timer.action = button:GetAttribute('action') or ActionButton_GetPagedID(button) or ActionButton_CalculateAction(button)
	end
end


local addonName = ...
local function CreateInterfacePanel() -- TODO: Clean this up
	--InterfaceOptions_AddCategory
	local frame = CreateFrame('frame')
	frame.name = addonName
	
	local title = frame:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	title:SetPoint('TOPLEFT', 16, -16)
	title:SetText(addonName .. ' Configuration')
	
	local function CreateTimer(name)
		local timer = CreateFrame('button', nil, frame, 'ActionButtonTemplate')
		timer:SetSize(36,36)
		timer.text = timer:CreateFontString(nil, 'OVERLAY')
		timer.text:SetFont(SETTINGS.font or STANDARD_TEXT_FONT, SETTINGS.size or 18, 'OUTLINE')
		timer.text:SetPoint('CENTER')
		timer.text:SetText('8m')
		
		timer.icon:SetTexture('interface/icons/inv_mushroom_11')
		
		timer.border = timer:CreateTexture(nil, 'OVERLAY')
		timer.border:SetTexture([[Interface\Buttons\CheckButtonHilight]])
		timer.border:SetAllPoints()
		timer.border:SetBlendMode('add')
		
		timer.subtext = timer:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
		timer.subtext:SetPoint('TOP', timer, 'BOTTOM', 0, -2)
		timer.subtext:SetText(name or '')
		return timer
	end
	
	local defaultTimer = CreateTimer('Default')
	defaultTimer:SetPoint('TOPLEFT', title, 'BOTTOMLEFT', 0, -10)
	
	local realTimer = CreateTimer('Real')
	realTimer:SetPoint('TOPLEFT', defaultTimer, 'TOPRIGHT', 20, 0)
	
	local energyTimer = CreateTimer('Energy')
	energyTimer:SetPoint('TOPLEFT', realTimer, 'TOPRIGHT', 20, 0)
	
	local auraTimer = CreateTimer('Aura')
	auraTimer:SetPoint('TOPLEFT', energyTimer, 'TOPRIGHT', 20, 0)
	
	local function colorCallback(restore)
		local newR, newG, newB, newA
		if restore then
			newR, newG, newB, newA = unpack(restore)
		else
			newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB()
		end
		return newR, newG, newB, newA
		--SETTINGS.colors.default.r, SETTINGS.colors.default.g, SETTINGS.colors.default.b, SETTINGS.colors.default.a = newR, newG, newB, newA
	end
	
	local function defaultCallback(restore)
		SETTINGS.colors.default.r, SETTINGS.colors.default.g, SETTINGS.colors.default.b, SETTINGS.colors.default.a = colorCallback(restore)
		defaultTimer.text:SetTextColor(SETTINGS.colors.default.r, SETTINGS.colors.default.g, SETTINGS.colors.default.b, SETTINGS.colors.default.a)
	end
	
	local function realCallback(restore)
		SETTINGS.colors.real.r, SETTINGS.colors.real.g, SETTINGS.colors.real.b, SETTINGS.colors.real.a = colorCallback(restore)
		realTimer.text:SetTextColor(SETTINGS.colors.real.r, SETTINGS.colors.real.g, SETTINGS.colors.real.b, SETTINGS.colors.real.a)
		realTimer.border:SetVertexColor(SETTINGS.colors.real.r, SETTINGS.colors.real.g, SETTINGS.colors.real.b, SETTINGS.colors.real.a)
		energyTimer.border:SetVertexColor(SETTINGS.colors.real.r, SETTINGS.colors.real.g, SETTINGS.colors.real.b, SETTINGS.colors.real.a)
	end
	
	local function energyCallback(restore)
		SETTINGS.colors.energy.r, SETTINGS.colors.energy.g, SETTINGS.colors.energy.b, SETTINGS.colors.energy.a = colorCallback(restore)
		energyTimer.text:SetTextColor(SETTINGS.colors.energy.r, SETTINGS.colors.energy.g, SETTINGS.colors.energy.b, SETTINGS.colors.energy.a)
		energyTimer.border:SetVertexColor(SETTINGS.colors.real.r, SETTINGS.colors.real.g, SETTINGS.colors.real.b, SETTINGS.colors.real.a)
	end
	
	local function auraCallback(restore)
		SETTINGS.colors.aura.r, SETTINGS.colors.aura.g, SETTINGS.colors.aura.b, SETTINGS.colors.aura.a = colorCallback(restore)
		auraTimer.text:SetTextColor(SETTINGS.colors.aura.r, SETTINGS.colors.aura.g, SETTINGS.colors.aura.b, SETTINGS.colors.aura.a)
		auraTimer.border:SetVertexColor(SETTINGS.colors.aura.r, SETTINGS.colors.aura.g, SETTINGS.colors.aura.b, SETTINGS.colors.aura.a)
	end
	
	local function ShowColorPicker(r, g, b, a, changedCallback)
		ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a
		ColorPickerFrame.previousValues = {r,g,b,a}
		ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = changedCallback, changedCallback, changedCallback
		ColorPickerFrame:SetColorRGB(r,g,b)
		ColorPickerFrame:Hide()
		ColorPickerFrame:Show()
	end
	
	local color = SETTINGS.colors.default
	defaultTimer.text:SetTextColor(color.r, color.g, color.b, color.a)
	defaultTimer.border:Hide()
	defaultTimer:SetScript('OnClick', function()
		ShowColorPicker(color.r, color.g, color.b, color.a, defaultCallback)
	end)
	
	local color = SETTINGS.colors.real
	realTimer.text:SetTextColor(color.r, color.g, color.b, color.a)
	realTimer.border:SetVertexColor(color.r, color.g, color.b, color.a)
	realTimer:SetScript('OnClick', function()
		ShowColorPicker(color.r, color.g, color.b, color.a, realCallback)
	end)
	
	energyTimer.border:SetVertexColor(SETTINGS.colors.energy.r,SETTINGS.colors.energy.g,SETTINGS.colors.energy.b,SETTINGS.colors.energy.a)
	local color = SETTINGS.colors.energy
	energyTimer.text:SetTextColor(color.r, color.g, color.b, color.a)
	energyTimer.border:SetVertexColor(SETTINGS.colors.real.r, SETTINGS.colors.real.g, SETTINGS.colors.real.b, SETTINGS.colors.real.a)
	energyTimer:SetScript('OnClick', function()
		ShowColorPicker(color.r, color.g, color.b, color.a, energyCallback)
	end)
	
	local color = SETTINGS.colors.aura
	auraTimer.text:SetTextColor(color.r, color.g, color.b, color.a)
	auraTimer.border:SetVertexColor(color.r, color.g, color.b, color.a)
	auraTimer:SetScript('OnClick', function()
		ShowColorPicker(color.r, color.g, color.b, color.a, auraCallback)
	end)
	
	local sizeSlider = CreateFrame('slider', 'CrackCooldownConfigSizeSlider', frame, 'OptionsSliderTemplate')
	sizeSlider:SetPoint('TOPLEFT', defaultTimer, 'BOTTOMLEFT', 0, -30)
	--CrackCooldownConfigSizeSliderText:SetText('Font Size')
	CrackCooldownConfigSizeSliderLow:SetText('Font Size')
	sizeSlider:SetScript('OnValueChanged', function(self, value)
		SETTINGS.size = value
		defaultTimer.text:SetFont(SETTINGS.font, SETTINGS.size, "OUTLINE")
		realTimer.text:SetFont(SETTINGS.font, SETTINGS.size, "OUTLINE")
		energyTimer.text:SetFont(SETTINGS.font, SETTINGS.size, "OUTLINE")
		auraTimer.text:SetFont(SETTINGS.font, SETTINGS.size, "OUTLINE")
		CrackCooldownConfigSizeSliderHigh:SetText(value)
	end)
	--sizeSlider:SetSize(100,20)
	sizeSlider:SetPoint('LEFT', sizeText, 'RIGHT', 5, 0)
	sizeSlider:SetMinMaxValues(12, 24)
	sizeSlider:SetValue(SETTINGS.size)
	CrackCooldownConfigSizeSliderHigh:SetText(SETTINGS.size)
	sizeSlider:SetValueStep(2)
	
	local glintCheckBox = CreateFrame('checkbutton', 'CrackCooldownConfigGlintCheckBox', frame, 'ChatConfigCheckButtonTemplate')
	glintCheckBox:SetPoint('LEFT', sizeSlider, 'RIGHT', 10, 0)
	CrackCooldownConfigGlintCheckBoxText:SetPoint('LEFT', CrackCooldownConfigGlintCheckBox, 'RIGHT', 5, 0)
	CrackCooldownConfigGlintCheckBoxText:SetText('Glint')
	glintCheckBox.tooltip = 'Sparkley thing'
	glintCheckBox:SetChecked(SETTINGS.glint)
	glintCheckBox:SetScript('OnClick', function(self) SETTINGS.glint = self:GetChecked() and true or false end)
	
	InterfaceOptions_AddCategory(frame)
	
	SLASH_CRACKCOOLDOWN1 = '/crackcooldown'
	function SlashCmdList.CRACKCOOLDOWN() InterfaceOptionsFrame_OpenToCategory(frame) end
end

f:SetScript('OnEvent', function(self, event, ...)
	if event == 'UNIT_AURA' or event == 'PLAYER_TARGET_CHANGED' then
		updateAuras()
	--elseif event == 'ACTIONBAR_SLOT_CHANGED' or event == 'ACTIONBAR_PAGE_CHANGED' then
		--recalculateActions()
	elseif event == 'PLAYER_LOGIN' then
		GetActionButtons(UIParent:GetChildren())
	elseif event == 'ADDON_LOADED' and ... == addonName then
		SETTINGS = CrackCooldownSettings or SETTINGS
		CrackCooldownSettings = SETTINGS
		CreateInterfacePanel()
		updateAuras()
		f:RegisterEvent('UNIT_AURA')
		f:RegisterEvent('PLAYER_TARGET_CHANGED')
		--f:RegisterEvent('ACTIONBAR_SLOT_CHANGED') -- looping through every action button on this event is overkill
		--f:RegisterEvent('ACTIONBAR_PAGE_CHANGED')
		f:RegisterEvent("PLAYER_LOGIN")
	end
end)

f:RegisterEvent('ADDON_LOADED')